from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os



#
# Sensor for a  binary clock.
#
class BinaryClock(Sensor):
	#This is attribut for the configuration of the desklet
    MODE_12H = "12h"
    MODE_24H = "24h"
    MODE_OCTAL = 8
    MODE_DECIMAL = 10
    MODE_HEXADECIMAL = 16
	#setting to 99 so all the led will appear on start.
    GLOBAL_HOUR = 99
    GLOBAL_TENHOUR = 99
    GLOBAL_MIN = 99
    GLOBAL_TENMIN = 99
    GLOBAL_SEC = 99
    GLOBAL_TENSEC = 99

    THEME_DARK = "dark"
    THEME_LIGHT = "light"
    BLUE_LED = "blue"
    RED_LED = "red"
    	#Set to this so the theme refresh at the start...
    GLOBAL_THEME = "reset_theme"
    GLOBAL_LED_THEME = "reset_theme"

	#
	#Initialisation of the sensor.
	#
    def __init__(self):
        global _; _ = i18n.Translator("binaryclock-sensor")
        Sensor.__init__(self)
	self._set_config_type("show-seconds", TYPE_BOOL, 1)
	self._set_config_type("mode", TYPE_STRING, self.MODE_24H)
	self._set_config_type("binary_mode", TYPE_INT, self.MODE_DECIMAL)
	self._set_config_type("theme",TYPE_STRING, self.THEME_DARK)
	self._set_config_type("led_theme",TYPE_STRING,self.BLUE_LED)
	self.__path = os.getcwd()
	self.add_timer(0, self.__get_data)
    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):

        configurator = self._new_configurator()
	configurator.set_name(_("BinaryClock"))
	configurator.add_title(_("Binary Clock Settings"))
	configurator.add_checkbox(_("_Show seconds"), "show-seconds",
                              _("Toggles the seconds on and off"))
        configurator.add_option(_("Display mode"), "mode",
                                _("12h or 24h display mode"),
                                [(_("12h"), self.MODE_12H),
                                 (_("24h"), self.MODE_24H)])
	configurator.add_option(_("Binary mode"), "binary_mode",
                                _("Octal, Decimal or Hexadecimal display"),
                                [(_("Octal"), self.MODE_OCTAL),
                                 (_("Decimal"), self.MODE_DECIMAL),
                                 (_("Hexadecimal"),self.MODE_HEXADECIMAL)])
	configurator.add_option(_("Theme"), "theme",
                                _("Choose for a light or dark theme to fit your desktop"),
                                [(_("Dark"), self.THEME_DARK),
                                 (_("Light"), self.THEME_LIGHT)])
        configurator.add_option(_("Theme for Led"), "led_theme",
                                _("Choose theme for the led"),
                                [(_("Blue Led"), self.BLUE_LED),
                                 (_("Red Led"), self.RED_LED)])


	
        return configurator

	#
	#Method call each second to refresh the desklet
	#
    def __get_data(self):
        output = self.new_output()
	show_secs = self.get_config("show-seconds")
	binary_mode = self.get_config("binary_mode")
	mode = self.get_config("mode")
	theme = self.get_config("theme")
	led_theme = self.get_config("led_theme")

	if(theme != self.GLOBAL_THEME):
		output.set("theme_right", os.path.join(self.__path, "icons", "theme", "right_%(theme)s.png" % vars()))
		output.set("theme_center", os.path.join(self.__path, "icons", "theme", "center_%(theme)s.png" % vars()))
		output.set("theme_left", os.path.join(self.__path, "icons", "theme", "left_%(theme)s.png" % vars()))
		self.GLOBAL_THEME = theme
	if(led_theme != self.GLOBAL_LED_THEME):
		self.GLOBAL_SEC = 99
		self.GLOBAL_TENSEC = 99
		self.GLOBAL_MIN = 99
		self.GLOBAL_TENMIN = 99
		self.GLOBAL_HOUR = 99
		self.GLOBAL_TENHOUR = 99
		self.GLOBAL_LED_THEME = led_theme
	
        current_time = time.localtime(time.time())

        hour = current_time[3]
        minute = current_time[4]
        second = current_time[5]
		
	#Determine what is the time to trace on the desklets
        sec = second % binary_mode
        tensec = second / binary_mode
	min = minute % binary_mode
	tenmin = minute / binary_mode
	if (mode == self.MODE_12H ):
		hour = hour % 12
	tenhour = hour / binary_mode
	hour = hour % binary_mode
	#Only if we want to show the second.
	if (show_secs):
             if(sec != self.GLOBAL_SEC):
	     	output.set("sec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(sec)s.png" % vars()))
		self.GLOBAL_SEC = sec
             if(tensec != self.GLOBAL_TENSEC):
	     	output.set("tensec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(tensec)s.png" % vars()))
		self.GLOBAL_TENSEC = tensec
	else:
	     if(sec != self.GLOBAL_SEC):
		     output.set("sec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "blank.png"))
		     self.GLOBAL_SEC = sec
             if(tensec != self.GLOBAL_TENSEC):
		     output.set("tensec", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "blank.png"))
		     self.GLOBAL_TENSEC = tensec
        #Show all the other led.
	if(min != self.GLOBAL_MIN):
		output.set("min", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(min)s.png" % vars()))
		self.GLOBAL_MIN = min
        if(tenmin != self.GLOBAL_TENMIN):
		output.set("tenmin", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenmin)s.png" % vars()))
		self.GLOBAL_TENMIN = tenmin
        if(hour != self.GLOBAL_HOUR):
		output.set("hour", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(hour)s.png" % vars()))
		self.GLOBAL_HOUR = hour
        if(tenhour != self.GLOBAL_TENHOUR):
		output.set("tenhour", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenhour)s.png" % vars()))
		self.GLOBAL_TENHOUR = tenhour
	#Output the result on the desklet
        self.send_output(output)
	#Reset the timer.
        self.add_timer(1000, self.__get_data)


	#This method is not used for the moment.  It's return the current date.
    def __get_date(self, current_time):

        day = current_time[2]
        month = current_time[1]
        year = current_time[0]
        date = "%(day)d.%(month)d.%(year)d" % vars()

        return date


def new_sensor(args): return apply(BinaryClock, args)
