import time
import os

from sensor.Sensor import Sensor
from main import admin

from utils import i18n
from utils.datatypes import *

from cal import cal
#from ev_editor import editor
from ev_list import events_list
from ev_viewer import event_viewer

class Calendar(Sensor):
	def __init__(self, *args):
		global _; _ = i18n.Translator("calendar-sensor")
		global gfx; gfx = os.getcwd() + "/gfx/"
		
		self.width = 195
	
	
		# call the constructor of the super class
		Sensor.__init__(self)

		# run a timeout function once a second
		
		self._set_config_type("first_weekday", TYPE_INT, 1)
		self._set_config_type("day_color", TYPE_STRING, "black")
		self._set_config_type("today_color", TYPE_STRING, "red")
		self._set_config_type("event_day_color", TYPE_STRING, "blue")
		self._set_config_type("dayname_font", TYPE_STRING, "Sans bold 10")
		self._set_config_type("dayname_color", TYPE_STRING, "black")
		self._set_config_type("day_font", TYPE_STRING, "Sans 10")

		self._add_timer(1000, self.create_display)
		self._add_timer(30000, self.update_display)

		self._watch_config(self.watch_config)
		
		self.event_views = {}
	
	def get_configurator(self):
	
		weekdays = [_("Sunday"), _("Monday")]
		
		cweekdays = []
		i = 0
		for x in weekdays:
			cweekdays.append([x, i])
			i += 1
		
		configurator = self._new_configurator()
		
		configurator.set_name(_("Calendar"))
		configurator.add_title(_("Calendar Settings"))
		configurator.add_option(_("First Weekday"), "first_weekday",
		                        _("The first day of the week."), cweekdays)
		configurator.add_font_selector(_("Daynames font:"), "dayname_font",
		                               _("The font for the day names."))
		configurator.add_color_selector(_("Daynames color:"), "dayname_color",
		                                _("The color of the names of the days."))
		configurator.add_font_selector(_("Days font:"), "day_font",
		                               _("The font for the days."))
		configurator.add_color_selector(_("Default color:"), "day_color",
		                                _("The color of the calendar days."))
		configurator.add_color_selector(_("Today's color:"), "today_color",
		                                _("The color of today."))
		configurator.add_color_selector(_("Event day color:"), "event_day_color",
		                                _("The color of days with events."))
		
		return configurator
	
	def watch_config(self, key, value):
		self.create_display()
	
	def create_display(self):
	
		self.c = cal(self._get_config("first_weekday"))
		self.page = self.c.get_page(time.localtime())
	
		out = self._new_output()
		
		out.set("winwidth", self.width)
		out.set("wincenter", self.width / 2)
		for x in range(7):
			out.set("dayname_color[%d]"%x, self._get_config("dayname_color"))
			out.set("dayname_font[%d]"%x, self._get_config("dayname_font"))
		
		for x in range(42):
			out.set("day_font[%d]"%x, self._get_config("day_font"))
		
		self._send_output(out)
		
		self.print_month()
		self.print_daynames()
		self.print_days()
		return 0
	
	def update_display(self):
		self.print_month()
		self.print_days()
		return 1
	
	def print_month(self):
		m,y = self.page
		out = self._new_output()
		out.set("month", self.c.get_monthname(m))
		out.set("monthx", self.width * .69)
		out.set("year", y)
		out.set("yearx", self.width * .71)
		self._send_output(out)
		return 1
		
	
	def print_daynames(self):
		output = self._new_output()
		for x in range(7):
			d = x + self._get_config("first_weekday")
			output.set("dayname[%d]"%x, self.c.next_dayname(d))
		self._send_output(output)

	def print_days(self):
		out = self._new_output()
		d = 0
		stop = 0
		w, ld = self.c.get_fweek_mdays(self.page)
		today = self.c.get_today(self.page)
		tasks = self.c.get_days_with_tasks(self.page)
		self.cur_date = {}
		w += 1 - self._get_config("first_weekday")
		if w < 0:
			w += 7
		for i in range(42):
			if i >= w:
				d += 1
			if d > ld:
				d = 0
				stop = 1
			if d <= 0 or stop == 1:
				out.set("day[%d]"%i, " ")
			else:
				out.set("day[%d]"%i, "%d"%d)
				self.cur_date[i] = self.page[1], self.page[0], d
				
			if d == today:
				out.set("dcol[%d]"%i, self._get_config("today_color"))
			elif tasks.count(d) > 0:
				out.set("dcol[%d]"%i, self._get_config("event_day_color"))
			else:
				out.set("dcol[%d]"%i, self._get_config("day_color"))
			
		self._send_output(out)
		
		return 1

	def call_action(self, action, path, args = []):
		if (action == "next"):
			self.page = self.c.page_increase(self.page)
			self.update_display()
		elif (action == "prev"):
			self.page = self.c.page_decrease(self.page)
			self.update_display()
		elif (action == "cur"):
			self.page = self.c.get_page(time.localtime())
			self.update_display()
		
		elif (action == "events"):
			#if path != []:
			evli = events_list(self.cur_date[self.last_path], self.update_display,
			                   self._get_config("first_weekday"))
			self.hide_events(self.last_path)
			
		elif (action == "enter_date"):
			if path != []:
				self.show_events(path[0])
		elif (action == "leave_date"):
			if path != []:
				self.hide_events(path[0])

	def show_events(self, path):
		self.last_path = path
		try:
			id = self._get_config_id()
			
			did = ""
			for c in id:
				if (c.isdigit()): 
					did += c
				else:
					break
			geo = admin.get_geometry(did)
			x = geo[0]
			y = geo[1] + geo[3]
			self.event_views[path] = event_viewer(self.cur_date[path], x, y)
			self.event_views[path].show()
		except KeyError:
			pass

	def hide_events(self, path):
		try:
			self.event_views[path].hide()
		except KeyError:
			pass
	
def new_sensor(args): 
	return apply(Calendar, args)
