
import xml.dom.minidom, xml.dom.ext
import os.path
import string

from event import event

class docdb:
	def __init__(self, filename="~/.gdesklets/calendar.xml"):
		self.filename = os.path.expanduser(filename)
		if os.path.exists(self.filename):
			self.doc = xml.dom.minidom.parse(self.filename)
			self.root = self.doc.documentElement
		else:
			self.doc = xml.dom.minidom.Document()
			self.root = self.doc.createElement("calendar")
			self.doc.appendChild(self.root)
	
	def get_last_id(self):
		e = self.root.getElementsByTagName("event")
		ID = 0
		for x in e:
			if x.getAttribute("ID") == '':
				continue
			xID = int(x.getAttribute("ID"))
			if xID > ID:
				ID = xID
		return ID
	
	def add_event(self, ev):
		e = self.doc.createElement("event")
		self.root.appendChild(e)

		if ev.ID == 0:
			ev.ID = self.get_last_id() + 1
		else:
			self.remove_event(ev.ID)

		e.setAttribute("ID", str(ev.ID))
		if ev.get_type() != 0 or ev.get_type() != "":
			e.setAttribute("type", ev.get_type())
		if ev.has_date == 1:
			e.setAttribute("date", string.strip(ev.date))
		if ev.full_day == 1:
			e.setAttribute("full_day", "true")
		else:
			e.setAttribute("start", string.strip(ev.start))
			e.setAttribute("end", string.strip(ev.end))

		if ev.has_reminder == 1:
			s = self.doc.createElement("reminder")
			e.appendChild(s)
			if ev.has_offset == 1 and ev.offset != 0:
				s.setAttribute("offset", ev.offset)

		if ev.has_summary == 1:
			s = self.doc.createElement("summary")
			e.appendChild(s)
			t = self.doc.createTextNode(string.strip(ev.summary))
			s.appendChild(t)
		
		if ev.has_note == 1:
			s = self.doc.createElement("note")
			e.appendChild(s)
			t = self.doc.createTextNode(string.strip(ev.note))
			s.appendChild(t)

		if ev.has_recurrence == 1:
			s = self.doc.createElement("recurrence")
			s.setAttribute("until", string.strip(ev.recurrence_until))
			e.appendChild(s)
			
			if ev.recurrence_weekdays != []:
				d = self.doc.createElement("weekdays")
				s.appendChild(d)
				t = self.doc.createTextNode(str(ev.recurrence_weekdays))
				d.appendChild(t)
			
			if ev.recurrence_exceptions != []:
				d = self.doc.createElement("exceptions")
				s.appendChild(d)
				t = self.doc.createTextNode(str(ev.recurrence_exceptions))
				d.appendChild(t)
			
		self.write()

	def get_event(self, evID):
		e = self.root.getElementsByTagName("event")
		for x in e:
			if evID == int(x.getAttribute("ID")):
				ev = event()
				
				ev.ID = int(x.getAttribute("ID"))
				ev.set_type(x.getAttribute("type"))
				ev.date = x.getAttribute("date")
				if x.getAttribute("full_day") == "true":
					ev.full_day = 1
					ev.start = "00:00"
					ev.end = "00:00"
				else:
					ev.start = x.getAttribute("start")
					ev.end = x.getAttribute("end")
				ev.summary = self.getText(x.getElementsByTagName("summary")[0].childNodes)
				ev.note = self.getText(x.getElementsByTagName("note")[0].childNodes)
				if x.getElementsByTagName("recurrence") != []:
					ev.has_recurrence = 1
					r = x.getElementsByTagName("recurrence")[0]
					ev.recurrence_until = r.getAttribute("until")
					if r.getElementsByTagName("weekdays") != []:
						t = self.getText(r.getElementsByTagName("weekdays")[0].childNodes)
						ev.recurrence_weekdays = eval(t)
					if r.getElementsByTagName("exceptions") != []:
						t = self.getText(r.getElementsByTagName("exceptions")[0].childNodes)
						ev.recurrence_exceptions = eval(t)
				return ev

	def remove_event(self, evID):
		e = self.root.getElementsByTagName("event")
		for x in e:
			if x.getAttribute("ID") == "":
				continue
			if evID == int(x.getAttribute("ID")):
				self.root.removeChild(x)
		self.write()

	def add_type(self, t):
		if t.name == "Default":
			return
		e = self.doc.createElement("type")
		e.setAttribute("name", string.strip(t.name))
		e.setAttribute("has_date", t.has_date)
		e.setAttribute("has_start", t.has_start)
		e.setAttribute("has_end", t.has_end)
		e.setAttribute("has_summary", self.has_summary)
		e.setAttribute("has_note", self.has_note)
		e.setAttribute("has_offset", self.has_offset)
		e.setAttribute("has_reminder", self.has_reminder)
		
		d = self.doc.createElement("defaults")
		e.appendChild(d)
		if t.has_default_date == 1:
			d.setAttribute("date", string.strip(t.date))
		if t.has_default_start == 1:
			d.setAttribute("start", string.strip(t.start))
		if t.has_default_end == 1:
			d.setAttribute("end", string.strip(t.end))

		if ev.has_reminder == 1:
			s = self.doc.createElement("reminder")
			e.appendChild(s)
			if ev.has_offset == 1:
				s.setAttribute("offset", ev.offset)

		if t.has_default_summary == 1:
			s = self.doc.createElement("summary")
			d.appendChild(s)
			t = self.doc.createTextNode(string.strip(t.summary))
			s.appendChild(t)
		if t.has_default_note == 1:
			s = self.doc.createElement("note")
			d.appendChild(s)
			t = self.doc.createTextNode(string.strip(ev.note))
			s.appendChild(t)
			
	
		self.write()
	
	def get_type_names(self):
		e = self.root.getElementsByTagName("type")
		names = ["Default"]
		for x in e:
			names.append(x.getAttribute("name"))
		return names
		
	
	def getText(self, nodelist):
		rc = ""
		for node in nodelist:
			if node.nodeType == node.TEXT_NODE:
				rc = rc + node.data
		return string.strip(rc)

	def get_events_from_date(self, date):
		e = self.root.getElementsByTagName("event")
		evs = []
		for x in e:
			ev = self.get_event(int(x.getAttribute("ID")))
			if ev.event_on_date(date) == 1:
				evs.append(ev)
		return evs
	
	def get_event_days_from_month(self, m, y):
		e = self.root.getElementsByTagName("event")
		days = []
		for x in e:
			ev = self.get_event(int(x.getAttribute("ID")))
			d = ev.event_in_month(m, y)
			if d != 0:
				for i in d:
					days.append(i)
		return days
	
	def write(self):
		xml.dom.ext.PrettyPrint(self.doc, open(self.filename, "w"))
