
import gtk
import time


class CalendarSelection(gtk.HBox):
	def __init__(self, date=None, firstweekday=1):
		gtk.HBox.__init__(self)
		self.entry = gtk.Entry()
		self.entry.show()
		arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_IN)
		self.button = gtk.Button()
		self.button.add(arrow)
		self.button.show_all()
		self.pack_start(self.entry, gtk.FALSE, gtk.FALSE)
		self.pack_start(self.button, gtk.FALSE, gtk.FALSE)
		
		self.button.connect("clicked", self.open_calendar)
		if date == None:
			self.set_date(time.localtime())
		else:
			self.set_date(date)
		
		self.firstweekday = firstweekday

		self.entry.connect("activate", self.set_date_from_entry)
			
		self.win_open = 0
		
	def open_calendar(self, widget, data=None):
		if self.win_open == 1:
			self.win.destroy()
			self.win_open = 0
			return
		self.win = gtk.Window(gtk.WINDOW_POPUP)
		self.win.set_decorated(gtk.FALSE)
		self.win.set_has_frame(gtk.TRUE)
		self.win.set_transient_for(widget.get_toplevel())
		self.win.set_destroy_with_parent(gtk.TRUE)
		
		x, y = self.entry.window.get_origin()
		geo = self.entry.window.get_geometry()
		self.win.move(x, y+geo[3])
		
		#vbox = gtk.VBox()

		calendar = gtk.Calendar()
		
		calendar.select_month(self.date[1]-1, self.date[0])
		calendar.select_day(self.date[2])
		if self.firstweekday == 1:
			calendar.display_options(gtk.CALENDAR_SHOW_HEADING + gtk.CALENDAR_SHOW_DAY_NAMES + gtk.CALENDAR_WEEK_START_MONDAY)
		else:
			calendar.display_options(gtk.CALENDAR_SHOW_HEADING + gtk.CALENDAR_SHOW_DAY_NAMES)

		
		self.win.add(calendar)

		#vbox.pack_start(calendar)
		
		#hbox = gtk.HBox()
		#ok = gtk.Button("Ok", gtk.STOCK_OK)
		#close = gtk.Button("Close", gtk.STOCK_CLOSE)
		#hbox.pack_end(close, gtk.FALSE, gtk.FALSE)
		#hbox.pack_end(ok, gtk.FALSE, gtk.FALSE)
		#vbox.pack_end(hbox)
		
		self.win.show_all()
		self.win_open = 1
		
		#ok.connect("clicked", self.day_selected, calendar, self.win)
		#close.connect("clicked", self.close_window, self.win)
		calendar.connect("day-selected", self.day_selected)
		calendar.connect("day-selected-double-click", self.day_selected_double_click)
		
	def day_selected_double_click(self, calendar, data=None):
		self.win.destroy()
		self.win_open = 0
		date = calendar.get_date()
		self.date = time.strptime(str(date[0]) + "-" + str(date[1]+1) + "-" + str(date[2]), "%Y-%m-%d")
		self.entry.set_text(time.strftime("%x", self.date))
		self.entry.activate()
	
	def day_selected(self, calendar, data=None):
		date = calendar.get_date()
		self.date = time.strptime(str(date[0]) + "-" + str(date[1]+1) + "-" + str(date[2]), "%Y-%m-%d")
		self.entry.set_text(time.strftime("%x", self.date))
		self.entry.activate()
		
	def close_window(self, widget, win):
		win.destroy()
		self.win_open = 0
		
	def get_date(self):
		d = time.strftime("%Y-%m-%d", self.date)
		return d
	
	def set_date(self, date):
		self.date = time.strptime(date, "%Y-%m-%d")
		self.entry.set_text(time.strftime("%x", self.date))
	
	def set_date_from_entry(self, entry, data=None):
		try:
			d = time.strptime(entry.get_text(), "%x")
			self.date = d
		except ValueError:
			entry.set_text(time.strftime("%x", self.date))
