## Updated version, without the threading error
#   - Changes by: Daniel Alves <daniel.alves@linux-portugal.org>

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import random
import os
import commands
import time

# This is for testing purposed
left   = 0
center = 1
right  = 2

# DeskQuotes, a sensor package to put random quotes on your desktop
class DeskQuote(Sensor):

	def __init__(self, sourcefile, textalignment, interval, width):
		global _; _ = i18n.Translator("deskquote-sensor")
        # instance variable for counting
        	self.__quote = ''
		self.__interval = int(interval)
		self.__need_run = 1

        # call the constructor of the super class
        	Sensor.__init__(self)
		self._set_config_type("update-frequency", TYPE_INT, self.__interval)
		self._set_config_type("update-width", TYPE_INT, int(width))
		self._set_config_type("quote-file", TYPE_STRING, sourcefile)
		self._set_config_type("text-alignment", TYPE_STRING, textalignment)
		self._add_timer(0, self.__on_timer)

	def get_configurator(self):
	        configurator = self._new_configurator()
        	configurator.set_name(_("Panel"))
        	configurator.add_title(_("Appearance"))
		configurator.add_spin(_("Refresh:"), "update-frequency",_("The update rate in seconds"),10, 1200)
		configurator.add_spin(_("Width:"), "update-width",_("The approximate width in characters"),20, 60)
		configurator.add_title(_("Quote File"))
		configurator.add_entry(_("Quote File"), "quote-file", _("The file to read quotes from"))
		configurator.add_title(_("Text Alignment"))
		configurator.add_option(_("Text Alignment"), "text-alignment",
                                _("Left, Right or Center alignment"),
                                [(_("Left"), "left"),
                                 (_("Center"), "center"), (_("Right"), "right")])
		
		return configurator

# A function to align text, from Magnus L. Hetland (found on usenet)

	def __align(self,line,width,alignment):
    		space = width - len(line)
    		if alignment == "center":
        		return ' '*(space/2) + line + \
               		' '*(space/2 + space%2)
    		elif alignment == "right":
        		return ' '*space + line
    		else:
        		return line + ' '*space

# On Timer function (Main function)
	def __on_timer(self):
		# Kill timer if interval was changed
		if ( self.__interval != int(self._get_config("update-frequency")) and self.__need_run == 0 ):
			self.__interval = int(self._get_config("update-frequency"))
			self._add_timer(self.__interval*1000, self.__on_timer)
			return 0
		# Continuing
		quote = ""
		aligned = ""
		self.__need_run = 0
		# Get filename and text width
		sourcefile = self._get_config("quote-file")
		linewidth = self._get_config("update-width")
		# Opens quotes file
		quotefile = open(sourcefile, 'r')
		# Read down and gather info as we go
		s = quotefile.readline()
		delimiter = s[0]
		s = quotefile.readline()
		s = quotefile.readline()
		chars = s.split()
		lines = int(chars[0])
		# Get a random line number
		line = random.randint(3, lines)
		# Go to the random line
		s = quotefile.readline()
		i = 3
		while ( i != line ):
			s = quotefile.readline()
			i = i + 1
		# Now read until delimiter is reached
		while ( s != delimiter + '\n'):
			s = quotefile.readline()
		# Now we are about to read the first line of our quote
		s = quotefile.readline()
		# Out string-length limiter, if it's too long we chop it
		# in two and then parse back for a space (so we don't chop any
		# words in half
		if (len(s) >= linewidth):
			length = len(s)
			string1 = s[0:linewidth]
			string2 = s[linewidth:length]
			spaced = linewidth - 1
			while ( string1[spacer] != ' ' ): spacer = spacer - 1
			# Feed the shortened strings into the quote
			quote = quote + string1[0:spacer+1] + "\n" + string1[spacer+1:46] + string2
		else: quote = quote + s
		# Now just recurse through the file until the next delimiter, doing the above.
		while ( s != delimiter + '\n'):
			s = quotefile.readline()
			if ( s != delimiter + '\n'):
				if ( len(s) >= linewidth):
					length = len(s)
					string1 = s[0:linewidth]
					string2 = s[linewidth:length]
					spacer = linewidth - 1
					while ( string1[spacer] != ' ' ): spacer = spacer - 1
					quote = quote + string1[0:spacer+1] + '\n' + string1[spacer+1:46] + string2
				else: quote = quote + s
		#end_while
		quote = quote + s
		# We now have our quote
		# Create a counter for the alignment step                        
		j = 0
		# And split the quote into seperate lines
                s = quote.splitlines()
		# While we are within the quote, pass the lines to the alignment function
                while (s[j] != delimiter):
                	aligned = aligned + "\n" + self.__align(s[j],linewidth,alignment=self._get_config("text-alignment"))
                	j = j + 1
		# Now we're ready to pass the data out to the display
		data = self._new_output()
		data.set("quote", aligned)
		self._send_output(data)
		quotefile.close()
		# Return true
		return 1

def new_sensor(args): return apply(DeskQuote, args)
