from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import os
import commands

class IconButton(Sensor):
	
	__ANIM_STEPS = 4
	
	def __init__(self,cmd,animation = "1", zoom = "1.6"):
		
		global _; _ = i18n.Translator("iconbutton-sensor")
		
		Sensor.__init__(self)
		self._set_config_type("animation", TYPE_BOOL, int(animation))
		self._set_config_type("ib_command", TYPE_STRING, cmd)
		
		self.__scaledzoom = float(zoom)
		self.__zoom = 1.0
		self.__counter = 0
		self.__change = 0
		self.__sign = 1
		
		self.__zoom_steps = (float(zoom) - 1.0) / self.__ANIM_STEPS
		
		self._add_timer(50, self.__animation)

	def __hilight(self):
		self.__zoom = 1.0
		self.__sign = 1
		self.__change = 1
		self.__counter = 0

	def __unhilight(self):
		self.__zoom = self.__scaledzoom
		self.__change = 1
		self.__counter = 0
		self.__sign = -1

	def __animation(self):
		if (self.__change):
			self.__change = 0
			if (self.__counter < self.__ANIM_STEPS):
				self.__zoom += self.__zoom_steps * self.__sign
				self.__counter += 1
				self.__change = 1
			
			output = self._new_output()
			output.set("icon-scale", self.__zoom)
			self._send_output(output)
		return 1

	def get_configurator(self):
		
		configurator = self._new_configurator()
		configurator.set_name(_("IconButton command"))
		configurator.add_title(_("Settings"))
		configurator.add_entry(_("Command"), "ib_command",
					      _("You can change the command used by the IconCommand sensor"))
		configurator.add_checkbox(_("Animation"), "animation",
					      _("Enables/Disables animation"))
		return configurator

	def call_action(self, action, path, args = []):
		
		# enter icon
		if (action == "enter"):
			if self._get_config("animation"):
				self.__hilight()
		
		# leave icon panel
		elif (action == "leave"):
			if self._get_config("animation"):
				self.__unhilight()
		
		# start application
		elif (action == "launch"):
			cmd = self._get_config("ib_command")
			os.system("cd ~ && " + cmd + " & disown")


def new_sensor(args): return apply(IconButton, args)
