from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import iCon

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class ImoncSensor(Sensor):

	def __init__(self, hostname = "horst", interval = "1000"):
		self.con = None
		global _; _ = i18n.Translator("imonc-sensor")

        	# call the constructor of the super class
	        Sensor.__init__(self)

		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("imond_host", TYPE_STRING, hostname)

		self._add_timer(int(interval), self.__my_timer)

	def __my_timer(self):
		if (self.con == None or self.con.connected == 0):
			# connect to router
			self.con = iCon.ImondConnection(self._get_config("imond_host"))
			self.con.connect()

		if (self.con.connected == 0):
			return 1

		output = self._new_output()
		output.set("hostname", self.con.get_hostname())
		output.set("version", self.con.get_version())
		output.set("uptime", self.con.get_uptime())
		output.set("date", self.con.get_date())
		output.set("cpu", self.con.get_cpu())

		inout = self.con.get_rate()
		output.set("in", human_readable(inout[0]))
		output.set("out", human_readable(inout[1]))
		output.set("inout", inout[0]+inout[1])

		self._send_output(output)

		return 1

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name("Imonc")
		configurator.add_title(_("Imonc Configuration"))
		configurator.add_entry(_("Hostname"), "imond_host", _("Remote host name"))

		configurator.add_spin(_("Update every (ms):"), "update",
					_("Updates imond info every x milliseconds"),
					100, 60000)
		return configurator

	def _shutdown(self):
		if (self.con != None):
			self.con.disconnect()
		print "disconnected"

def new_sensor(args): return apply(ImoncSensor, args)
