from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os

import libdesklets

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value
		
class Memory(Sensor):

	def __init__(self, target = "RAM", interval = "1000"):
		
		if target =="RAM" or target =="Mem":
			target = 0
		else :
			target = 1
		
		global _; _ = i18n.Translator("memory-sensor")
		
		self.__free = 0
		self.__used = 0
		self.__total = 0
		self.__percent = 0
		self.__targets = []
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("target", TYPE_INT, target)
		
		self._add_timer(int(interval), self.__on_tick)
		self._add_thread(self.__command_thread)
		
		self.__targets.append(["RAM",0])
		self.__targets.append(["Swap",1])

	def __on_tick(self):
		data = self._new_output()
		
		data.set("total", human_readable(self.__total))
		data.set("used", human_readable(self.__used))
		data.set("free", human_readable(self.__free))
		data.set("percent", self.__percent)
		if self._get_config("target") == 0:
			data.set("name","RAM")
		else :
			data.set("name","Swap")
			
		self._send_output(data)
		
		return 1

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Memory"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ms):"), "update",
					_("Updates the CPU load every x milliseconds"),
					100, 60000)
		configurator.add_option(_("Target:"), "target",
					_("Possible targets"),self.__targets)
		return configurator
	
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			
			if self._get_config("target") == 0:
				(total,used) = libdesklets.memory.get_ram()
			else :
				(total,used) = libdesklets.memory.get_swap()
				
			self.__total = total
			self.__used = used
			self.__free = self.__total - self.__used
			self.__percent = int(100*self.__used/self.__total)
			
			time.sleep(self._get_config("update")*0.001)
				


def new_sensor(args): return apply(Memory, args)
