"""
__init__.py
"""

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n, dialog

import os.path
import Queue
import time

from filter import Filter
from screenbuffer import ScreenBuffer
from tailbook import TailBook
import watchlist

__author__ = "Bas van Oostveen"
__name__ = "MultiTail"
__version__ = "0.2.3"
__license__ = "GPL"

class config:
	update = 5
	length = 5
	scrollspeed = 0
	showintro = True
	linecount = True
	font = "Courier New 14"
	direct = 0
	crop = 0
	wrap = 0
	wrapat = 0
	watches = []
	_watches = "/var/log/messages"
	colors = []
	_colors = "#000000"
	errors = []
	_errors = "ok"
	
class MultiTail(Sensor):
	def __init__(self, *args):
		""" init """
		global _; _ = i18n.Translator("MultiTail")
		
		# vars
		self.__queue = Queue.Queue(64)
		
		# init superclass
		Sensor.__init__(self)
		
		# set config types
		self._set_config_type("update", TYPE_INT, config.update)
		self._set_config_type("length", TYPE_INT, config.length)
		self._set_config_type("scrollspeed", TYPE_INT, config.scrollspeed)
		self._set_config_type("showintro", TYPE_BOOL, config.showintro)
		self._set_config_type("linecount", TYPE_BOOL, config.linecount)
		self._set_config_type("font", TYPE_STRING, config.font)
		self._set_config_type("direct", TYPE_BOOL, config.direct)
		self._set_config_type("crop", TYPE_INT, config.crop)
		self._set_config_type("wrap", TYPE_INT, config.wrap)
		self._set_config_type("wrapat", TYPE_INT, config.wrapat)
		self._set_config_type("watches", TYPE_LIST, config._watches)
		self._set_config_type("colors", TYPE_LIST, config._colors)
		self._set_config_type("errors", TYPE_LIST, config._errors)
		
		# init screenbuffer
		self.__sbuffer = ScreenBuffer(self._get_config)
		
		# init filter
		self.__filter = Filter(self._get_config)
		
		# register
		self._add_timer(0, self.__applet_init)
		self._add_timer(1000, self.__applet_update)
		self._add_thread(self.__command_thread)
		self._watch_config(self.__config_changed)
	
	def __config_changed(self, key, value):
		""" update config object """
		try:
			setattr(config, key, value)
		except e:
			print e
				
		if key in ["length", "wrapat"]:
			output = self._new_output()
			output.set(key, value)
			self._send_output(output)
		
		if key in ["font"]:
			output = self._new_output()
			for c in xrange(config.length):
				output.set("font[%d]" % c, config.font)
			self._send_output(output)
		
	def __applet_init(self):
		# reread config
		for key in ["update", "length", "showintro", "font", "wrap", "wrapat",
					"direct", "linecount", "watches", "colors", "errors"]:
			setattr(config, key, self._get_config(key))
		
		# show intro
		if config.showintro:
			self.__sbuffer.showIntro()
		
		# initial applet
		output = self._new_output()
		for key in ["length", "wrapat"]:
			output.set(key, getattr(config, key))
		
		for c in xrange(config.length):
			output.set("font[%d]" % c, config.font)
			
		self._send_output(output)
		
		# never again
		return 0
	
	def __applet_update(self):
		length = self._get_config("length")
		
		output = self._new_output()
		output.set("length", length)
		
		if self._get_config("linecount"):
			for c in xrange(length):
				output.set("linecount[%d]" % c, c)
		else:
			for c in xrange(length):
				output.set("linecount[%d]" % c, '')
		
		ss = self._get_config("scrollspeed")
		qs = self.__queue.qsize()
		if qs==0 and config.direct==0:
			ss = 0
		elif config.direct==1:
			ss = qs
		elif ss==0:
			ss = (qs/length) and length-(length/4) or qs
		
		for c in xrange(ss):
			try:
				color, name, line = self.__queue.get(False)
			except Queue.Empty:
				break
				
			for l in self.__filter(line):
				self.__sbuffer.put(color, name, l)
		
		if self.__sbuffer.isChanged():
			for c in xrange(length):
				color, name, line = self.__sbuffer.get(c)
				output.set("name[%d]" % c, name)
				output.set("line[%d]" % c, line)
				output.set("linecolor[%d]" % c, color)
		
		self._send_output(output)
		
		return True
	
	def __command_thread(self):
		tick = 1
		tailbook = TailBook()
		while (True):
			if (self._is_stopped()): break
			
			# check tailbook
			tailbook.check(self._get_config("watches"), self._get_config("colors"))
			
			# build pool
			pool = tailbook.getList()
			
			# run pool
			while (len(pool)>0):
				for p in pool:
					error, line = tailbook.getLine(p)
					if line=="":
						del pool[pool.index(p)]
						break
					if error:
						try:
							i = self._get_config("watches").index(p)
							e = self._get_config("errors")
							e[i] = line
							self._set_config("errors", e)
						except:
							pass
					qb = (tailbook.getColor(p), "["+tailbook.getName(p)+"]", line)
					self.__queue.put(qb)
				time.sleep(0.1) # microsleep
			
			time.sleep(tick)

	def __watch_dialog(self, src, *args):
		if (watchlist.win==None):
			watchlist.show(self._get_config, self._set_config)
		else:
			dialog.warning("Already open", "Watch list already open")
	
	def get_configurator(self):
		""" create configurator """
		c = self._new_configurator()
		c.set_name(_(__name__))
		c.add_title(_("%s version %s" % (__name__,__version__)))
		
		c.add_title(_("Display"))
		c.add_spin(	_("Scroll speed:"), "scrollspeed",
					_("Number of new lines to scroll per second (0 = automatic)"), 0, 127)
		c.add_checkbox(_("Direct/Fast mode (disables scroll speed)"), "direct",
					   _("Push new lines to the desklet as fast as possible"))
		c.add_checkbox(_("Show intro"), "showintro", _("Show Intro"))
		c.add_checkbox(_("Linecount"), "linecount", _("Number all the lines"))
		c.add_spin(_("Lines on screen:"), "length",
				   _("Number of lines to display on screen"), 1, 127)
		
		c.add_title(_("Font"))
		c.add_font_selector(_("Font"), "font", _("The font to use."))
				   
		c.add_title(_("Width"))
		c.add_spin(_("Crop:"), "crop",
				   _("Maximum number of chars in one line (0 = disabled"), 0, 256)
		c.add_spin(_("Wrap:"), "wrap",
				   _("Maximum number of chars on one line (0 = disabled"), 0, 256)
		c.add_spin(	_("Label Wrap:"), "wrapat",
					_("Label wrap feature of gdesklets, set in pixels"), 0, 1024)
		
		return c
	
	def call_action(self, action, path, args=[]):
		""" react to user events/actions """
		
		# open menu
		if (action == "menu"):
			self._open_menu([
				("Watch logs...", 1, None, self.__watch_dialog, [])
			])
	
	def _shutdown(self):
		self._set_config("watches", config.watches)
		self._set_config("colors", config.colors)

def new_sensor(args): return apply(MultiTail, args)
