"""
TailBook.py
"""

from xml.sax.saxutils import escape
import os.path

class Tail:
	alive = False
	fn = None
	path = ""
	name = ""
	error = ""
	color = "#000000"

class TailBook:
	def __init__(self):
		self.__files = {}
		
	def __open(self, path):
		t = Tail()
		t.active = True
		t.path = path
		t.name = os.path.basename(path)
		try:
			f = open(path, "r")
			f.seek(os.path.getsize(path))
			t.fn = f
		except Exception, e:
			t.error = "%s: '%s'" % (e.strerror, e.filename)
			
		self.__files[path] = t

	def __close(self, path):
		if self.__files.has_key(path):
			t = self.__files[path]
			try:
				t.fn.close()
			except:
				pass
			del self.__files[path]
	
	def check(self, list, colors):
		assert len(list)==len(colors)
		
		# open new
		for n in list:
			if not self.__files.has_key(n):
				self.__open(n)
				
		# close old 
		for o in self.__files.keys():
			if not o in list:
				self.__close(o)
		
		# change colors
		for i in range(len(list)):
			key = list[i]
			t = self.__files[key]
			if not t.color == colors[i]:
				t.color = colors[i]
				self.__files[key]=t
	
	def getList(self):
		return self.__files.keys()
	
	def getName(self, key):
		if self.__files.has_key(key):
			return self.__files[key].name
		return ""
	
	def getColor(self, key):
		if self.__files.has_key(key):
			return self.__files[key].color
		return ""
	
	def getLine(self, key):
		if self.__files.has_key(key):
			t = self.__files[key]
			
			if not t.active:
				return False, ""
				
			if t.active and not t.error=="":
				t.active=False
				self.__files[key] = t
				return True, t.error
				
			try:
				return False, escape(t.fn.readline().replace("\n","").replace('\r',''))
			except Exception, e:
				t.active = False
				t.error = "%s: '%s'" % (e.strerror, e.filename)
				self.__files[key] = t
				return True, t.error
