from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import libdesklets as lib

import os
import time
import Queue


class SysInfoReader(Sensor):


    def __init__(self):

        global _; _ = i18n.Translator("sysinfo-sensor")

        # the maximum number of fields
        self.__array_size = 5

        self.__fonts = ""

        self.__backgr = ""

        self.__key_color = ""
        self.__val_color = ""

        self.__current_size = 0
        self.__size_changed = 0

        self.__speed_in_max  = 0
        self.__speed_out_max = 0

        self.__path  = os.path.join(os.getcwd(), "gfx")
        self.__queue = Queue.Queue(1)

        Sensor.__init__(self)

        self._set_config_type("fonts", TYPE_STRING, "Sans 10")
        self._set_config_type("key_color", TYPE_STRING, "white")
        self._set_config_type("val_color", TYPE_STRING, "yellow")
        self._set_config_type("background", TYPE_STRING, "gfx/bg.png")
        self._set_config_type("memory", TYPE_STRING, "RAM")
        self._set_config_type("partition", TYPE_STRING, "/")
        self._set_config_type("network", TYPE_STRING, "eth0")

        self._add_thread(self.__get_sys_info_thread)
        self._add_timer(100, self.__get_sys_info)



    def get_configurator(self):

        part = []
        for dev, mpoint in lib.disk.get_partitions():
            part.append([dev, mpoint])

        net = []
        for dev in lib.net.get_devices():
            net.append([dev, dev])

        mem = [['RAM', 'RAM'], ['Swap', 'Swap']]

        configurator = self._new_configurator()

        configurator.set_name(_("System Information Desklet"))
        configurator.add_title(_("Settings"))
        configurator.add_file_selector(_("Background:"), "background",
                                       _("Selects the background image file."))
        configurator.add_font_selector(_("Font:"), "fonts", _("The font which "
                                       "is used to display the labels"))
        configurator.add_color_selector(_("Key color:"), "key_color",
                                        _("The color of the key"))
        configurator.add_color_selector(_("Value color:"), "val_color",
                                        _("The color of the value"))
        configurator.add_option(_("Partition:"), "partition",
                                "Specify partition type", part)
        configurator.add_option(_("Memory:"), "memory",
                                "Specify whether RAM or Swap should be "
                                "displayed", mem)
        configurator.add_option(_("Network:"), "network",
                                "Specify network device", net)

        return configurator



    def __do_sys(self, signal, index):

        os                             = lib.sys.get_os()
        arch                           = lib.sys.get_arch()
        name                           = lib.sys.get_name()
        kernel                         = lib.sys.get_kernel()
        l1                             = lib.sys.get_load_avg_1m()
        l5                             = lib.sys.get_load_avg_5m()
        l15                            = lib.sys.get_load_avg_15m()
        tasks, running                 = lib.sys.get_tasks()
        users                          = lib.sys.get_users()
        uptime                         = lib.sys.get_uptime()
        d, h, m, s                     = lib.convert.secs_to_dhms(uptime)
        idletime                       = lib.sys.get_idletime()
        idle_d, idle_h, idle_m, idle_s = lib.convert.secs_to_dhms(idletime)
        idle_p                         = (idletime / float(uptime)) * 100

        signal.set("value1[%(index)d]"    % vars(), os + " @ " + name)
        signal.set("value2[%(index)d]"    % vars(), kernel[:6])
        signal.set("value3[%(index)d]"    % vars(), str(l1) + " " + str(l5) + \
                    " " + str(l15))
        signal.set("value4[%(index)d]"    % vars(), users)
        signal.set("value5[%(index)d]"    % vars(), str(d) + "d " + \
                    str(h) + "h " + str(m) + "m ")
        signal.set("value6[%(index)d]"    % vars(), str(idle_d) + "d " + \
                    str(idle_h) + "h " + str(idle_m) + "m ")
        signal.set("fill[%(index)d]"      % vars(), "%.2f" % idle_p)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("show5[%(index)d]"     % vars(), 1)
        signal.set("show6[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_cpu(self, signal, index):

        cpu_type    = lib.cpu.get_model()
        cpu_speed   = lib.cpu.get_speed()
        cpu_cache   = lib.cpu.get_cache_size()
        total, load = lib.cpu.get_load()

        signal.set("value1[%(index)d]"    % vars(), cpu_type)
        signal.set("value2[%(index)d]"    % vars(), cpu_speed)
        signal.set("value3[%(index)d]"    % vars(), cpu_cache)
        signal.set("fill[%(index)d]"      % vars(), load)
        signal.set("plot[%(index)d]"      % vars(), load)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 1)



    def __do_ram(self, signal, index):

        mem = self._get_config("memory")
        if (mem == "RAM"):
            total, used = lib.memory.get_ram()
        else:
            total, used = lib.memory.get_swap()

        if (total != 0):
           free = total - used
           perc = (used / float(total)) * 100
        else:
            free = 0
            perc = 0

        signal.set("value1[%(index)d]"    % vars(), mem)
        signal.set("value2[%(index)d]"    % vars(),
                    lib.convert.human_readable_bytes(total))
        signal.set("value3[%(index)d]"    % vars(),
                    lib.convert.human_readable_bytes(used))
        signal.set("value4[%(index)d]"    % vars(),
                    lib.convert.human_readable_bytes(free))
        signal.set("fill[%(index)d]"      % vars(), "%.2f" % perc)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_hdd(self, signal, index):

        part                = self._get_config("partition")
        fs_type             = lib.disk.get_fs(part)
        hdd_total, hdd_used = lib.disk.get_size(part)
        hdd_free            = hdd_total - hdd_used
        hdd_perc            = (hdd_used / hdd_total) * 100

        signal.set("value1[%(index)d]"    % vars(), part + " (" + fs_type + ")")
        signal.set("value2[%(index)d]"    % vars(),
                   lib.convert.human_readable_bytes(hdd_total))
        signal.set("value3[%(index)d]"    % vars(),
                   lib.convert.human_readable_bytes(hdd_used))
        signal.set("value4[%(index)d]"    % vars(),
                   lib.convert.human_readable_bytes(hdd_free))
        signal.set("fill[%(index)d]"      % vars(), "%.2f" % hdd_perc)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_net(self, signal, index):

        net                 = self._get_config("network")
        ipaddr              = lib.net.get_ipaddr(net)
        pack_in, pack_out   = lib.net.get_pack_in_out(net)
        bytes_in, bytes_out = lib.net.get_bytes_in_out(net)
        speed_in, speed_out = lib.net.get_speed_in_out(net)

        signal.set("value1[%(index)d]"    % vars(), net)
        signal.set("value2[%(index)d]"    % vars(), ipaddr)
        signal.set("value3[%(index)d]"    % vars(),
                   lib.convert.human_readable_bytes(bytes_in))
        signal.set("value4[%(index)d]"    % vars(),
                   lib.convert.human_readable_bytes(bytes_out))
        signal.set("value5[%(index)d]"    % vars(), pack_in)
        signal.set("value6[%(index)d]"    % vars(), pack_out)
        signal.set("plot1[%(index)d]"     % vars(), speed_in)
        signal.set("plot2[%(index)d]"     % vars(), speed_out)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("show5[%(index)d]"     % vars(), 1)
        signal.set("show6[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 0)
        signal.set("showplot[%(index)d]"  % vars(), 1)



    def __update_config(self):

        index = 0

        fonts      = self._get_config("fonts")
        key_color  = self._get_config("key_color")
        val_color  = self._get_config("val_color")
        background = self._get_config("background")

        v1 = (self.__fonts, self.__key_color, self.__val_color, self.__backgr)
        v2 = (fonts, key_color, val_color, background)

        # only take action when some value has changed
        if (v1 != v2 or self.__size_changed):
            signal = self._new_output()

            signal.set("icon[%(index)d]" % vars(), "gfx/Info.png")
            signal.set("key1[%(index)d]" % vars(), _(""))
            signal.set("key2[%(index)d]" % vars(), _("Kernel:"))
            signal.set("key3[%(index)d]" % vars(), _("Load:"))
            signal.set("key4[%(index)d]" % vars(), _("Users:"))
            signal.set("key5[%(index)d]" % vars(), _("Uptime:"))
            signal.set("key6[%(index)d]" % vars(), _("Idle time:"))
            index += 1
            signal.set("icon[%(index)d]" % vars(), "gfx/cpu.png")
            signal.set("key1[%(index)d]" % vars(), _(""))
            signal.set("key2[%(index)d]" % vars(), _("Clock:"))
            signal.set("key3[%(index)d]" % vars(), _("Cache:"))
            signal.set("key4[%(index)d]" % vars(), _("Idle time:"))
            index += 1
            signal.set("icon[%(index)d]" % vars(), "gfx/memory.png")
            signal.set("key1[%(index)d]" % vars(), _(""))
            signal.set("key2[%(index)d]" % vars(), _("Total:"))
            signal.set("key3[%(index)d]" % vars(), _("Used:"))
            signal.set("key4[%(index)d]" % vars(), _("Free:"))
            index += 1
            signal.set("icon[%(index)d]" % vars(), "gfx/hdd.png")
            signal.set("key1[%(index)d]" % vars(), _(""))
            signal.set("key2[%(index)d]" % vars(), _("Total:"))
            signal.set("key3[%(index)d]" % vars(), _("Used:"))
            signal.set("key4[%(index)d]" % vars(), _("Free:"))
            index += 1
            signal.set("icon[%(index)d]" % vars(), "gfx/network.png")
            signal.set("key1[%(index)d]" % vars(), _(""))
            signal.set("key2[%(index)d]" % vars(), _("IP addr:"))
            signal.set("key3[%(index)d]" % vars(), _("Bytes in:"))
            signal.set("key4[%(index)d]" % vars(), _("Bytes out:"))
            signal.set("key5[%(index)d]" % vars(), _("Packets in:"))
            signal.set("key6[%(index)d]" % vars(), _("Packets out:"))
            index += 1

            if (self.__backgr != background):
                signal.set("background", background)
            if (self.__fonts != fonts or self.__size_changed):
                for i in xrange(self.__array_size):
                    signal.set("fonts[%(i)d]" %vars(), fonts)
            if (self.__key_color != key_color or self.__size_changed):
                for i in xrange(self.__array_size):
                    signal.set("key_color[%(i)d]" %vars(), key_color)
            if (self.__val_color != val_color or self.__size_changed):
                for i in xrange(self.__array_size):
                    signal.set("val_color[%(i)d]" %vars(), val_color)

            self._send_output(signal)

            self.__fonts        = fonts
            self.__key_color    = key_color
            self.__val_color    = val_color
            self.__backgr       = background
            self.__size_changed = 0



    def __get_sys_info(self):

        self.__update_config()
        if (not self.__queue.empty()):
            signal = self.__queue.get()
            self._send_output(signal)

            size = signal.get("size")
            if (size != self.__current_size):
                self.__current_size = size
                self.__size_changed = 1

        return 1



    def __get_sys_info_thread(self):

        while(1):
            index = 0

            if (self._is_stopped()): break

            signal = self._new_output()

            try:
                self.__do_sys(signal, index)
                index += 1
                self.__do_cpu(signal, index)
                index += 1
                self.__do_ram(signal, index)
                index += 1
                self.__do_hdd(signal, index)
                index += 1
                self.__do_net(signal, index)
                index += 1
            except:
                import traceback; traceback.print_exc()

            signal.set("size", index)

            self.__queue.put(signal)
            time.sleep(1)



def new_sensor(args): return apply(SysInfoReader, args)
