from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import commands
import re
import os
import time
import Queue
import urllib

# Sensor for retrieving trafic information.
class SytadinSensor(Sensor):

	TRAFFIC_SOURCE = "http://www.sytadin.tm.fr/cgi-bin/cltcgi_calcul"
	
	# regular expressions to parse the traffic data
	RE_TEMPS = re.compile("(?P<duree>\d+) mn")
	RE_DISTANCE = re.compile("(?P<distance>\d+).(?P<distanceb>\d+) km")
	RE_FIABILITE = re.compile("(?P<fiabilite>\d+) %")

	def __init__(self):

		global _; _ = i18n.Translator("sytadin-sensor")

		self.__queue = Queue.Queue(1)

		Sensor.__init__(self)

		self._set_config_type("depart", TYPE_STRING, "BP Issy")
		self._set_config_type("arrivee", TYPE_STRING, "BP Italie")
		self._set_config_type("t_update", TYPE_STRING, "5")

		self._add_thread(self.__get_traffic_thread)
		self._add_timer(0, self.__get_traffic)
	
	# Returns the configuration GUI.
	def get_configurator(self):

		configurator = self._new_configurator()
		configurator.set_name(_("Configuration de Sytadin"))
		configurator.add_title(_("Paramètres du trajet:"))
		configurator.add_entry(_("Départ"), "depart", _("Le nom de votre point de départ"))
		configurator.add_entry(_("Arrivée"), "arrivee", _("Le nom "
						"de votre point d'arrivée"))
		configurator.add_entry(_("Rafraîchir (min):"), "t_update",
					_("Intervalle de mise à jour "
						"du trajet en minutes"))
		return configurator

	def __get_traffic(self):

		# check the queue regularly
		if (not self.__queue.empty()):
			output = self.__queue.get()
			self._send_output(output)
			
		self._add_timer(500, self.__get_traffic)

	# Thread function fo retrieving the traffic data since this may take
	# time and must not block the application.
	def __get_traffic_thread(self):

		counter = 0
		old_values = ("", "", "")
		current_time = int(time.time())
		# Set us back t_update seconds so the if statement will be true
		t_update = int(self._get_config("t_update"))
		last_update = current_time - (t_update * 60) - 1

		while(1):
			if (self._is_stopped()): break

			depart = (self._get_config("depart"))
			arrivee = self._get_config("arrivee")
			t_update = int(self._get_config("t_update"))
			webdepart = urllib.quote(depart)
			webarrivee = urllib.quote(arrivee)

			# Initialize the current time.
			current_time = int(time.time())

			if((depart, arrivee, t_update) != old_values or \
			last_update + (t_update * 60) <= current_time):
				old_values = (webdepart, webarrivee, t_update)
				current_time = int(time.time())
				last_update = int(time.time())

				params = "depart=%(webdepart)s&arrivee=%(webarrivee)s" % vars()
				fh = urllib.urlopen(self.TRAFFIC_SOURCE, params)
				data = fh.read()

				output = self._new_output()
				output.set("dep", depart)
				output.set("arr", arrivee)

				try:
					temps = \
					int(self.RE_TEMPS.search(data).group("duree"))
					distance = \
					int(self.RE_DISTANCE.search(data).group("distance"))
					distanceb = \
					int(self.RE_DISTANCE.search(data).group("distanceb"))
					fiabilite = \
					int(self.RE_FIABILITE.search(data).group("fiabilite"))
					output.set("temps", temps)
					output.set("distance", distance)
					output.set("distanceb", distanceb)
					output.set("fiabilite", fiabilite)

				except:
					output.set("temps", "Erreur")
					output.set("distance", "Erreur")
					output.set("distanceb", "")
					output.set("fiabilite", "Erreur")

				self.__queue.put(output)
			# end if
			time.sleep(1)
		# end while

def new_sensor(args): return apply(SytadinSensor, args)
