from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import commands
import time
import socket

#
# Sensor for retrieving hdd temperature information
#
class HddtempSensor(Sensor):

    def __init__(self):

        global _; _ = i18n.Translator("hddtemp-sensor")

        self.__hdd = "dev/hda"
        self.__hddlist = []
        self.__name = ""
        self.__ip = "127.0.0.1"        
        self.__port = ""        
        self.__temp = 0

        Sensor.__init__(self)

        self._set_config_type("hdd", TYPE_STRING, "/dev/hda")
        self._set_config_type("ip", TYPE_STRING, "127.0.0.1")                    
        self._set_config_type("port", TYPE_INT, 7634)
        self._set_config_type("update", TYPE_INT, 1)

        self._add_timer(1000, self.__get_temp)
        self._add_thread(self.__get_temp_thread)

    def get_configurator(self):
        configurator = self._new_configurator()
        configurator.set_name(_("Hddtemp"))
        configurator.add_title(_("Display Configuration"))
        configurator.add_spin(_("Update every (min):"), "update",
                              _("Updates the hdd temp every x min"),
                              1, 1440)
        configurator.add_option(_("HDD:"), "hdd",
                                _("Possible hdds"),self.__hddlist)
        configurator.add_entry(_("IP:"), "ip", _("The hddtemp server's IP"))
        configurator.add_spin(_("Port:"), "port",
                              _("The hddtemp server's port"),
                              1, 65535)
        return configurator
        
    def __get_temp(self):
        output = self._new_output()

        output.set("hdd", self.__hdd)
        output.set("temp", self.__temp)

        self._send_output(output)

        return 1

    def __get_temp_thread(self):

        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.connect(("", 7634))
        data_hdd = sock.recv(1024, socket.MSG_WAITALL)
        time.sleep(1)
        data_hdd = data_hdd.replace("|", "", 1)

        for disk in data_hdd.split("||"):            
            value = disk.split("|")
            self.__hddlist.append([value[0],value[0]])
            if(value[0] == self._get_config("hdd")):
                self.__hdd  = value[0] 
                self.__temp = value[2]
        sock.close()
    
        while (1):
            if (self._is_stopped()): break

            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((self._get_config("ip"), self._get_config("port")))
            data_hdd = sock.recv(1024, socket.MSG_WAITALL)
            time.sleep(1)
            data_hdd = data_hdd.replace("|", "", 1)

            for disk in data_hdd.split("||"):            
                value = disk.split("|")
                if(value[0] == self._get_config("hdd")):
                    self.__hdd  = value[0] 
                    self.__temp = value[2]
            sock.close()
        
            #end if
            time.sleep(self._get_config("update")*60)
        #end while
            

def new_sensor(args): return apply(HddtempSensor, args)
